@@ -11,12 +11,17 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: ean13.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1 - Author: cooldude2k $
+    $FileInfo: ean13.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
 import re, upcean.encode.getsfname, upcean.support;
-from PIL import Image, UnidentifiedImageError;
+try:
+ from PIL import Image, UnidentifiedImageError;
+ hasuie = True;
+except ImportError:
+ from PIL import Image;
+ hasuie = False;
 try:
  from io import StringIO, BytesIO;
 except ImportError:
@@ -48,19 +53,35 @@ def decode_ean13_barcode(infile="./ean13.png",resize=1,barheight=(48, 54),barwid
  else:
   try:
    infile.seek(0);
-   try:
-    upc_img = Image.open(infile).convert('RGB');
-   except UnidentifiedImageError:
-    return False;
-    '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
+   if(hasuie):
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except UnidentifiedImageError:
+     return False;
+     '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
+   else:
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except IOError:
+     return False;
+     '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
   except AttributeError:
-   try:
-    upc_img = Image.open(infile).convert('RGB');
-   except UnidentifiedImageError:
-    return False;
-    '''prefile = open(infile, "rb");
-    upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
-    prefile.close();'''
+   if(hasuie):
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except UnidentifiedImageError:
+     return False;
+     '''prefile = open(infile, "rb");
+     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
+     prefile.close();'''
+   else:
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except IOError:
+     return False;
+     '''prefile = open(infile, "rb");
+     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
+     prefile.close();'''
  barsize = barwidth[0] * int(resize);
  starty = (int(upc_img.size[1] / 2) - ((barwidth[1] - 1) * 9) ) + shiftxy[1];
  fist_number_dict = { 'LLLLLL': "0", 'LLGLGG': "1", 'LLGGLG': "2", 'LLGGGL': "3", 'LGLLGG': "4", 'LGGLLG': "5", 'LGGGLL': "6", 'LGLGLG': "7", 'LGLGGL': "8", 'LGGLGL': "9" };
@@ -113,12 +134,29 @@ def decode_ean13_barcode(infile="./ean13.png",resize=1,barheight=(48, 54),barwid
  endx = (42 + 42) * (barwidth[0] * int(resize));
  if(locatebarcode):
   prestartx = startx - (3 * (barwidth[0] * int(resize)));
+  jumpcodeend = jumpcode + (4 * (barwidth[0] * int(resize)));
+  endx = startx + ((42 + 4 + 42) * (barwidth[0] * int(resize)));
   postendx = endx + (3 * (barwidth[0] * int(resize)));
-  return ("ean18", prestartx, startx, 13, endx, postendx);
+  countyup = starty;
+  while(countyup >= 0):
+   curonepixel = upc_img.getpixel((prestartx, countyup));
+   curtwopixel = upc_img.getpixel((prestartx + (1 * (barwidth[0] * int(resize))), countyup));
+   if(curonepixel==barcolor[2] or curtwopixel==barcolor[0]):
+    break;
+   countyup += 1;
+  countyup -= 1;
+  countydown = starty;
+  while(countydown <= upc_img.size[1]):
+   curonepixel = upc_img.getpixel((prestartx, countydown));
+   curtwopixel = upc_img.getpixel((prestartx + (1 * (barwidth[0] * int(resize))), countydown));
+   if(curonepixel==barcolor[2] or curtwopixel==barcolor[0]):
+    break;
+   countydown -= 1;
+  countydown -= 1;
+  return ("ean18", prestartx, startx, jumpcode, jumpcodeend, endx, countyup, round(countyup / 2), round(countydown * 2), countydown, countyup, 13);
  startxalt = 0;
  listcount = 0;
  pre_upc_whole = [];
- prestartx = startx;
  while(startxalt < endx):
   listcount = 0;
   pre_upc_list = [];
@@ -127,8 +165,22 @@ def decode_ean13_barcode(infile="./ean13.png",resize=1,barheight=(48, 54),barwid
     startx += 5 * (barwidth[0] * int(resize));
    curpixel = upc_img.getpixel((startx, starty));
    if(curpixel==barcolor[0]):
+    incount = 0;
+    inbarwidth = barwidth[0] - 1;
+    while(incount<=inbarwidth):
+     incurpixel = upc_img.getpixel((startx + incount, starty));
+     if(incurpixel!=barcolor[0]):
+      return False;
+     incount += 1;
     pre_upc_list.append("1");
    if(curpixel==barcolor[2]):
+    incount = 0;
+    inbarwidth = barwidth[0] - 1;
+    while(incount<=inbarwidth):
+     incurpixel = upc_img.getpixel((startx + incount, starty));
+     if(incurpixel!=barcolor[2]):
+      return False;
+     incount += 1;
     pre_upc_list.append("0");
    startx += 1 * (barwidth[0] * int(resize));
    startxalt += 1 * (barwidth[0] * int(resize));
