@@ -11,12 +11,17 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: itf.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1 - Author: cooldude2k $
+    $FileInfo: itf.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
 import re, upcean.encode.getsfname, upcean.support;
-from PIL import Image, UnidentifiedImageError;
+try:
+ from PIL import Image, UnidentifiedImageError;
+ hasuie = True;
+except ImportError:
+ from PIL import Image;
+ hasuie = False;
 try:
  from io import StringIO, BytesIO;
 except ImportError:
@@ -48,19 +53,35 @@ def decode_itf_barcode(infile="./itf.png",resize=1,barheight=(48, 54),barwidth=(
  else:
   try:
    infile.seek(0);
-   try:
-    upc_img = Image.open(infile).convert('RGB');
-   except UnidentifiedImageError:
-    return False;
-    '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
+   if(hasuie):
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except UnidentifiedImageError:
+     return False;
+     '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
+   else:
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except IOError:
+     return False;
+     '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
   except AttributeError:
-   try:
-    upc_img = Image.open(infile).convert('RGB');
-   except UnidentifiedImageError:
-    return False;
-    '''prefile = open(infile, "rb");
-    upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
-    prefile.close();'''
+   if(hasuie):
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except UnidentifiedImageError:
+     return False;
+     '''prefile = open(infile, "rb");
+     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
+     prefile.close();'''
+   else:
+    try:
+     upc_img = Image.open(infile).convert('RGB');
+    except IOError:
+     return False;
+     '''prefile = open(infile, "rb");
+     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
+     prefile.close();'''
  threewidebar = True;
  barsize = barwidth[0] * int(resize);
  starty = (int(upc_img.size[1] / 2) - ((barwidth[1] - 1) * 9) ) + shiftxy[1];
@@ -178,6 +199,8 @@ def decode_itf_barcode(infile="./itf.png",resize=1,barheight=(48, 54),barwidth=(
      break;
   barcodesize = int((barcodesize) / 2);
  endx = int(startx + ( (barcodesize * 18 ) * (barwidth[0] * int(resize)) ));
+ if(locatebarcode):
+  endx -= (1 * (barwidth[0] * int(resize)));
  if(threewidebar):
   if(locatebarcode):
    postendx = endx + (5 * (barwidth[0] * int(resize)));
@@ -188,28 +211,88 @@ def decode_itf_barcode(infile="./itf.png",resize=1,barheight=(48, 54),barwidth=(
   endx = int(startx + ( (barcodesize * 14 ) * (barwidth[0] * int(resize)) ));
  if(locatebarcode):
   prestartx = startx - (4 * (barwidth[0] * int(resize)));
-  return ("itf", prestartx, startx, (barcodesize * 2), endx, postendx);
+  endx -= (1 * (barwidth[0] * int(resize)));
+  countyup = starty;
+  while(countyup >= 0):
+   curonepixel = upc_img.getpixel((prestartx, countyup));
+   curtwopixel = upc_img.getpixel((prestartx + (1 * (barwidth[0] * int(resize))), countyup));
+   if(curonepixel==barcolor[2] or curtwopixel==barcolor[0]):
+    break;
+   countyup += 1;
+  countyup -= 1;
+  countydown = starty;
+  while(countydown <= upc_img.size[1]):
+   curonepixel = upc_img.getpixel((prestartx, countydown));
+   curtwopixel = upc_img.getpixel((prestartx + (1 * (barwidth[0] * int(resize))), countydown));
+   if(curonepixel==barcolor[2] or curtwopixel==barcolor[0]):
+    break;
+   countydown -= 1;
+  countydown -= 1;
+  return ("itf", prestartx, startx, 0, 0, endx, countyup, round(countyup / 2), round(countydown * 2), countydown, countyup, (barcodesize * 2));
  listcount = 0;
  pre_upc_whole_left = [];
  pre_upc_whole_right = [];
  pre_upc_list_left = [];
  pre_upc_list_right = [];
  skiptwo = False;
- prestartx = startx;
  while(startx < endx):
   listcount = 0;
   curpixel = upc_img.getpixel((startx, starty));
   if(curpixel==barcolor[0]):
+   incount = 0;
+   inbarwidth = barwidth[0] - 1;
+   while(incount<=inbarwidth):
+    incurpixel = upc_img.getpixel((startx + incount, starty));
+    if(incurpixel!=barcolor[0]):
+     return False;
+    incount += 1;
    nexpixel = upc_img.getpixel((startx + (1 * (barwidth[0] * int(resize))), starty));
    if(nexpixel==barcolor[0] and startx<(endx - (2* (barwidth[0] * int(resize))) + 1)):
+    incount = 0;
+    inbarwidth = barwidth[0] - 1;
+    while(incount<=inbarwidth):
+     incurpixel = upc_img.getpixel(((startx + (1 * (barwidth[0] * int(resize)))) + incount, starty));
+     if(incurpixel!=barcolor[0]):
+      return False;
+     incount += 1;
+    if(threewidebar):
+     incount = 0;
+     inbarwidth = barwidth[0] - 1;
+     while(incount<=inbarwidth):
+      incurpixel = upc_img.getpixel(((startx + (2 * (barwidth[0] * int(resize)))) + incount, starty));
+      if(incurpixel!=barcolor[0]):
+       return False;
+      incount += 1;
     pre_upc_list_left.append("1");
     skiptwo = True;
    else:
     pre_upc_list_left.append("0");
     skiptwo = False;
   if(curpixel==barcolor[2]):
+   incount = 0;
+   inbarwidth = barwidth[0] - 1;
+   while(incount<=inbarwidth):
+    incurpixel = upc_img.getpixel((startx + incount, starty));
+    if(incurpixel!=barcolor[2]):
+     return False;
+    incount += 1;
    nexpixel = upc_img.getpixel((startx + (1 * (barwidth[0] * int(resize))), starty));
    if(nexpixel==barcolor[2] and startx<(endx - (2* (barwidth[0] * int(resize))) + 1)):
+    incount = 0;
+    inbarwidth = barwidth[0] - 1;
+    while(incount<=inbarwidth):
+     incurpixel = upc_img.getpixel(((startx + (1 * (barwidth[0] * int(resize)))) + incount, starty));
+     if(incurpixel!=barcolor[2]):
+      return False;
+     incount += 1;
+    if(threewidebar):
+     incount = 0;
+     inbarwidth = barwidth[0] - 1;
+     while(incount<=inbarwidth):
+      incurpixel = upc_img.getpixel(((startx + (2 * (barwidth[0] * int(resize)))) + incount, starty));
+      if(incurpixel!=barcolor[2]):
+       return False;
+      incount += 1;
     pre_upc_list_right.append("1");
     skiptwo = True;
    else:
