@@ -11,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: ean2.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1 - Author: cooldude2k $
+    $FileInfo: ean2.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -130,6 +130,14 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
  if(cairosupport and (imageoutlib=="cairo" or imageoutlib=="cairosvg")):
   if(outfileext=="SVG"):
    upc_preimg = cairo.SVGSurface(None, (29 * barwidth[0]), barheight[1] + (9 * barwidth[1]));
+  elif(outfileext=="PDF"):
+   upc_preimg = cairo.PDFSurface(None, (29 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
+  elif(outfileext=="PS" or outfileext=="EPS"):
+   upc_preimg = cairo.PSSurface(None, (29 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
+   if(outfileext=="EPS"):
+    upc_preimg.set_eps(True);
+   else:
+    upc_preimg.set_eps(False);
   else:
    upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (29 * barwidth[0]), barheight[1] + (9 * barwidth[1]));
   upc_img = cairo.Context (upc_preimg);
@@ -142,14 +150,14 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
  if(hidetext):
   LineSize = barheight[1];
  upc_array['code'].append( [0, 1, 0, 1, 1] );
- start_barcolor = [0, 1, 0, 1, 1];
+ start_barcode = [0, 1, 0, 1, 1];
  LineStart = 0;
  BarNum = 0;
- start_bc_num_end = len(start_barcolor);
+ start_bc_num_end = len(start_barcode);
  while(BarNum < start_bc_num_end):
-  if(start_barcolor[BarNum]==1):
+  if(start_barcode[BarNum]==1):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[0], imageoutlib);
-  if(start_barcolor[BarNum]==0):
+  if(start_barcode[BarNum]==0):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[2], imageoutlib);
   LineStart += barwidth[0];
   BarNum += 1;
@@ -233,15 +241,26 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
   scaler.scale(1/int(resize),1/int(resize));
   upc_imgpat.set_matrix(scaler);
   upc_imgpat.set_filter(cairo.FILTER_NEAREST);
-  if(outfileext=="SVG"):
+  if(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS"):
    if(outfile is None):
-    svgoutfile = None;
+    imgoutfile = None;
    else:
     if(sys.version[0]=="2"):
-     svgoutfile = StringIO();
+     imgoutfile = StringIO();
     if(sys.version[0]>="3"):
-     svgoutfile = BytesIO();
-   new_upc_preimg = cairo.SVGSurface(svgoutfile, (29 * barwidth[0]) * int(resize), (barheight[1] + (9 * barwidth[1])) * int(resize));
+     imgoutfile = BytesIO();
+   if(outfileext=="SVG"):
+    new_upc_preimg = cairo.SVGSurface(imgoutfile, ((29 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+   elif(outfileext=="PDF"):
+    new_upc_preimg = cairo.PDFSurface(imgoutfile, ((29 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+   elif(outfileext=="PS" or outfileext=="EPS"):
+    new_upc_preimg = cairo.PSSurface(imgoutfile, ((29 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+    if(outfileext=="EPS"):
+     new_upc_preimg.set_eps(True);
+    else:
+     new_upc_preimg.set_eps(False);
+   else:
+    new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (29 * barwidth[0]) * int(resize), (barheight[1] + (9 * barwidth[1])) * int(resize));
   else:
    new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (29 * barwidth[0]) * int(resize), (barheight[1] + (9 * barwidth[1])) * int(resize));
   new_upc_img = cairo.Context(new_upc_preimg);
@@ -292,13 +311,13 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -341,13 +360,13 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -382,11 +401,11 @@ def create_ean2_barcode_supplement(upc,outfile="./ean2_supplement.png",resize=1,
      with open(outfile, 'wb+') as f:
       f.write(new_upc_preimg.get_data().tobytes());
      return True;
-    elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+    elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
      new_upc_preimg.flush();
      new_upc_preimg.finish();
-     svgoutfile.seek(0);
-     svgouttext = svgoutfile.read();
+     imgoutfile.seek(0);
+     svgouttext = imgoutfile.read();
      with open(outfile, 'wb+') as f:
       f.write(svgouttext);
      return True;
@@ -455,13 +474,13 @@ def create_ean2_barcode(upc,outfile="./ean2.png",resize=1,hideinfo=(False, False
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -504,13 +523,13 @@ def create_ean2_barcode(upc,outfile="./ean2.png",resize=1,hideinfo=(False, False
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -545,11 +564,11 @@ def create_ean2_barcode(upc,outfile="./ean2.png",resize=1,hideinfo=(False, False
      with open(outfile, 'wb+') as f:
       f.write(new_upc_preimg.get_data().tobytes());
      return True;
-    elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+    elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
      new_upc_preimg.flush();
      new_upc_preimg.finish();
-     svgoutfile.seek(0);
-     svgouttext = svgoutfile.read();
+     imgoutfile.seek(0);
+     svgouttext = imgoutfile.read();
      with open(outfile, 'wb+') as f:
       f.write(svgouttext);
      return True;
