@@ -11,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: ean8.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1 - Author: cooldude2k $
+    $FileInfo: ean8.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -162,6 +162,14 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
  if(cairosupport and (imageoutlib=="cairo" or imageoutlib=="cairosvg")):
   if(outfileext=="SVG"):
    upc_preimg = cairo.SVGSurface(None, (83 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
+  elif(outfileext=="PDF"):
+   upc_preimg = cairo.PDFSurface(None, (83 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
+  elif(outfileext=="PS" or outfileext=="EPS"):
+   upc_preimg = cairo.PSSurface(None, (83 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
+   if(outfileext=="EPS"):
+    upc_preimg.set_eps(True);
+   else:
+    upc_preimg.set_eps(False);
   else:
    upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, (83 * barwidth[0]) + addonsize, barheightadd + (9 * barwidth[1]));
   upc_img = cairo.Context (upc_preimg);
@@ -172,10 +180,10 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
  upc_array = { 'upc': upc, 'code': [ ] };
  upc_array['code'].append( [0, 0, 0, 0, 0, 0, 0, 0, 0] );
  upc_array['code'].append( [1, 0, 1] );
- start_barcolor = [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1];
+ start_barcode = [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1];
  LineStart = 0;
  BarNum = 0;
- start_bc_num_end = len(start_barcolor);
+ start_bc_num_end = len(start_barcode);
  LineSize = barheight[1];
  while(BarNum < start_bc_num_end):
   if(BarNum<12):
@@ -184,9 +192,9 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
    LineSize = barheight[0];
   if(hidetext):
    LineSize = barheight[1];
-  if(start_barcolor[BarNum]==1):
+  if(start_barcode[BarNum]==1):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[0], imageoutlib);
-  if(start_barcolor[BarNum]==0):
+  if(start_barcode[BarNum]==0):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[2], imageoutlib);
   LineStart += barwidth[0];
   BarNum += 1;
@@ -303,14 +311,14 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
    InnerUPCNum += 1;
   NumZero += 1;
  upc_array['code'].append( [0, 1, 0, 1, 0] );
- mid_barcolor = [0, 1, 0, 1, 0];
+ mid_barcode = [0, 1, 0, 1, 0];
  mid_bc_num = 0;
- mid_bc_num_end = len(mid_barcolor);
+ mid_bc_num_end = len(mid_barcode);
  LineSize = barheight[1];
  while(mid_bc_num < mid_bc_num_end):
-  if(mid_barcolor[mid_bc_num]==1):
+  if(mid_barcode[mid_bc_num]==1):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[0], imageoutlib);
-  if(mid_barcolor[mid_bc_num]==0):
+  if(mid_barcode[mid_bc_num]==0):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[2], imageoutlib);
   mid_bc_num += 1;
   LineStart += barwidth[0];
@@ -354,9 +362,9 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
   NumZero += 1;
  upc_array['code'].append( [1, 0, 1] );
  upc_array['code'].append( [0, 0, 0, 0, 0, 0, 0, 0, 0] );
- end_barcolor = [1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0];
+ end_barcode = [1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0];
  end_bc_num = 0;
- end_bc_num_end = len(end_barcolor);
+ end_bc_num_end = len(end_barcode);
  LineSize = barheight[1];
  while(end_bc_num < end_bc_num_end):
   if(end_bc_num<4):
@@ -365,9 +373,9 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
    LineSize = barheight[0];
   if(hidetext):
    LineSize = barheight[1];
-  if(end_barcolor[end_bc_num]==1):
+  if(end_barcode[end_bc_num]==1):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[0], imageoutlib);
-  if(end_barcolor[end_bc_num]==0):
+  if(end_barcode[end_bc_num]==0):
    drawColorLine(upc_img, LineStart, 10, LineStart, LineSize, barwidth[0], barcolor[2], imageoutlib);
   end_bc_num = 1 + end_bc_num;
   LineStart += barwidth[0];
@@ -383,15 +391,26 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
   scaler.scale(1/int(resize),1/int(resize));
   upc_imgpat.set_matrix(scaler);
   upc_imgpat.set_filter(cairo.FILTER_NEAREST);
-  if(outfileext=="SVG"):
+  if(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS"):
    if(outfile is None):
-    svgoutfile = None;
+    imgoutfile = None;
    else:
     if(sys.version[0]=="2"):
-     svgoutfile = StringIO();
+     imgoutfile = StringIO();
     if(sys.version[0]>="3"):
-     svgoutfile = BytesIO();
-   new_upc_preimg = cairo.SVGSurface(svgoutfile, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+     imgoutfile = BytesIO();
+   if(outfileext=="SVG"):
+    new_upc_preimg = cairo.SVGSurface(imgoutfile, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+   elif(outfileext=="PDF"):
+    new_upc_preimg = cairo.PDFSurface(imgoutfile, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+   elif(outfileext=="PS" or outfileext=="EPS"):
+    new_upc_preimg = cairo.PSSurface(imgoutfile, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
+    if(outfileext=="EPS"):
+     new_upc_preimg.set_eps(True);
+    else:
+     new_upc_preimg.set_eps(False);
+   else:
+    new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
   else:
    new_upc_preimg = cairo.ImageSurface(cairo.FORMAT_RGB24, ((83 * barwidth[0]) + addonsize) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize));
   new_upc_img = cairo.Context(new_upc_preimg);
@@ -474,13 +493,13 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -523,13 +542,13 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
       stdoutfile.write(new_upc_preimg.get_data().tobytes());
       stdoutfile.seek(0);
       return stdoutfile;
-     elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+     elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
       new_upc_preimg.flush();
       new_upc_preimg.finish();
-      svgoutfile.seek(0);
-      svgouttext = svgoutfile.read();
+      imgoutfile.seek(0);
+      svgouttext = imgoutfile.read();
       stdoutfile.write(svgouttext);
-      svgoutfile.close();
+      imgoutfile.close();
       stdoutfile.seek(0);
       return stdoutfile;
      else:
@@ -564,11 +583,11 @@ def create_ean8_barcode(upc,outfile="./ean8.png",resize=1,hideinfo=(False, False
      with open(outfile, 'wb+') as f:
       f.write(new_upc_preimg.get_data().tobytes());
      return True;
-    elif(outfileext=="SVG" or imageoutlib=="cairosvg"):
+    elif(outfileext=="SVG" or outfileext=="PDF" or outfileext=="PS" or outfileext=="EPS" or imageoutlib=="cairosvg"):
      new_upc_preimg.flush();
      new_upc_preimg.finish();
-     svgoutfile.seek(0);
-     svgouttext = svgoutfile.read();
+     imgoutfile.seek(0);
+     svgouttext = imgoutfile.read();
      with open(outfile, 'wb+') as f:
       f.write(svgouttext);
      return True;
@@ -593,3 +612,12 @@ def draw_gtin8_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48
 
 def draw_ucc8_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=(1, 1),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)),imageoutlib="pillow"):
  return create_ucc8_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
+
+def encode_ean8_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=(1, 1),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)),imageoutlib="pillow"):
+ return create_ean8_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
+
+def encode_gtin8_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=(1, 1),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)),imageoutlib="pillow"):
+ return create_gtin8_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
+
+def encode_ucc8_barcode(upc,resize=1,hideinfo=(False, False, False),barheight=(48, 54),barwidth=(1, 1),textxy=(1, 1, 1),barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)),imageoutlib="pillow"):
+ return create_ucc8_barcode(upc,None,resize,hideinfo,barheight,barwidth,textxy,barcolor,imageoutlib);
