@@ -12,7 +12,7 @@
     Copyright 2011-2023 Game Maker 2k - http://intdb.sourceforge.net/
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: precairo.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1  - Author: cooldude2k $
+    $FileInfo: precairo.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1  - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -85,11 +85,9 @@ def drawText( ctx, size, x, y, text, ftype = "ocrb"  ):
  ctx.select_font_face( "Monospace" );
  ctx.set_font_size( size );
  fo = cairo.FontOptions();
- fo.set_antialias(cairo.ANTIALIAS_NONE);
- '''
- fo.set_hint_style(cairo.HINT_STYLE_NONE);
- fo.set_hint_metrics(cairo.HINT_METRICS_OFF);
- '''
+ fo.set_antialias(cairo.ANTIALIAS_DEFAULT);
+ fo.set_hint_style(cairo.HINT_STYLE_FULL);
+ fo.set_hint_metrics(cairo.HINT_METRICS_ON);
  ctx.set_font_options(fo);
  ctx.move_to( point1[0], point1[1] );
  ctx.show_text( text );
