@@ -11,7 +11,7 @@
     Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
     Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski
 
-    $FileInfo: support.py - Last Update: 4/1/2023 Ver. 2.8.14 RC 1 - Author: cooldude2k $
+    $FileInfo: support.py - Last Update: 4/7/2023 Ver. 2.9.0 RC 1 - Author: cooldude2k $
 '''
 
 from __future__ import absolute_import, division, print_function, unicode_literals;
@@ -205,6 +205,16 @@ def get_pillow_version(infotype=None):
    pillow_info = {'pillow_ver': pillow_ver, 'pil_is_pillow': pil_is_pillow};
    return pillow_info.get(infotype, pillow_info);
 
+def get_cairo_version(infotype=None):
+ cairosupport = check_for_cairo();
+ if(not cairosupport):
+  return cairosupport;
+ if(cairosupport):
+  import cairo;
+  cairo_ver = cairo.version;
+  cairo_info = {'cairo_ver': cairo_ver};
+  return cairo_info.get(infotype, cairo_info);
+
 linuxdist = None;
 try:
  linuxdist = platform.linux_distribution();
